package kr.co.vividnext.sodalive.admin.audition

import com.fasterxml.jackson.databind.ObjectMapper
import kr.co.vividnext.sodalive.admin.audition.role.AdminAuditionRoleRepository
import kr.co.vividnext.sodalive.audition.AuditionStatus
import kr.co.vividnext.sodalive.aws.s3.S3Uploader
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.fcm.FcmEvent
import kr.co.vividnext.sodalive.fcm.FcmEventType
import kr.co.vividnext.sodalive.utils.generateFileName
import org.springframework.beans.factory.annotation.Value
import org.springframework.context.ApplicationEventPublisher
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import org.springframework.web.multipart.MultipartFile

@Service
class AdminAuditionService(
    private val s3Uploader: S3Uploader,
    private val objectMapper: ObjectMapper,
    private val repository: AdminAuditionRepository,
    private val roleRepository: AdminAuditionRoleRepository,
    private val applicationEventPublisher: ApplicationEventPublisher,

    @Value("\${cloud.aws.s3.bucket}")
    private val bucket: String
) {
    @Transactional
    fun createAudition(image: MultipartFile, requestString: String) {
        val request = objectMapper.readValue(requestString, CreateAuditionRequest::class.java)
        val audition = repository.save(request.toAudition())

        val fileName = generateFileName("audition")
        val imagePath = s3Uploader.upload(
            inputStream = image.inputStream,
            bucket = bucket,
            filePath = "audition/production/${audition.id}/$fileName"
        )
        audition.imagePath = imagePath
    }

    @Transactional
    fun updateAudition(image: MultipartFile?, requestString: String) {
        val request = objectMapper.readValue(requestString, UpdateAuditionRequest::class.java)
        val audition = repository.findByIdOrNull(id = request.id)
            ?: throw SodaException("잘못된 요청입니다.\n다시 시도해 주세요.")

        if (request.title != null) {
            audition.title = request.title
        }

        if (request.information != null) {
            audition.information = request.information
        }

        if (request.isAdult != null) {
            audition.isAdult = request.isAdult
        }

        if (request.status != null) {
            if (
                (audition.status == AuditionStatus.COMPLETED || audition.status == AuditionStatus.IN_PROGRESS) &&
                request.status == AuditionStatus.NOT_STARTED
            ) {
                throw SodaException("모집전 상태로 변경할 수 없습니다.")
            }

            audition.status = request.status
        }

        if (request.originalWorkUrl != null) {
            audition.originalWorkUrl = request.originalWorkUrl
        }

        if (image != null) {
            val fileName = generateFileName("audition")
            val imagePath = s3Uploader.upload(
                inputStream = image.inputStream,
                bucket = bucket,
                filePath = "audition/production/${audition.id}/$fileName"
            )
            audition.imagePath = imagePath
        }

        if (request.isActive != null) {
            audition.isActive = request.isActive
        }

        if (request.status != null && request.status == AuditionStatus.IN_PROGRESS && audition.isActive) {
            applicationEventPublisher.publishEvent(
                FcmEvent(
                    type = FcmEventType.IN_PROGRESS_AUDITION,
                    title = "새로운 오디션 등록!",
                    message = "'${audition.title}'이 등록되었습니다. 지금 바로 오리지널 오디오 드라마 오디션에 지원해보세요!",
                    isAuth = audition.isAdult,
                    auditionId = audition.id ?: -1
                )
            )
        }
    }

    fun getAuditionList(offset: Long, limit: Long): GetAuditionListResponse {
        val totalCount = repository.getAuditionListCount()
        val items = repository.getAuditionList(offset = offset, limit = limit)
        return GetAuditionListResponse(totalCount, items)
    }

    fun getAuditionDetail(auditionId: Long): GetAuditionDetailResponse {
        val auditionDetail = repository.getAuditionDetail(auditionId = auditionId)
        val roleList = roleRepository.getAuditionRoleListByAuditionId(auditionId = auditionId)

        return GetAuditionDetailResponse(
            id = auditionDetail.id,
            title = auditionDetail.title,
            imageUrl = auditionDetail.imageUrl,
            information = auditionDetail.information,
            originalWorkUrl = auditionDetail.originalWorkUrl,
            roleList = roleList
        )
    }
}
