package kr.co.vividnext.sodalive.admin.content.curation

import kr.co.vividnext.sodalive.admin.content.AdminContentRepository
import kr.co.vividnext.sodalive.admin.content.series.AdminContentSeriesRepository
import kr.co.vividnext.sodalive.admin.content.tab.AdminContentMainTabRepository
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.main.curation.AudioContentCuration
import kr.co.vividnext.sodalive.content.main.curation.AudioContentCurationItem
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class AdminContentCurationService(
    private val repository: AdminContentCurationRepository,
    private val contentMainTabRepository: AdminContentMainTabRepository,
    private val seriesRepository: AdminContentSeriesRepository,
    private val contentRepository: AdminContentRepository,
    private val contentCurationItemRepository: AdminContentCurationItemRepository
) {
    @Transactional
    fun createContentCuration(request: CreateContentCurationRequest) {
        val tab = contentMainTabRepository.findByIdOrNull(request.tabId)
            ?: throw SodaException("잘못된 요청입니다.")

        val curation = AudioContentCuration(
            title = request.title,
            description = request.description,
            isAdult = request.isAdult,
            isSeries = request.isSeries
        )
        curation.tab = tab

        repository.save(curation)
    }

    @Transactional
    fun updateContentCuration(request: UpdateContentCurationRequest) {
        val audioContentCuration = repository.findByIdOrNull(id = request.id)
            ?: throw SodaException("잘못된 요청입니다.")

        if (request.title != null) {
            audioContentCuration.title = request.title
        }

        if (request.description != null) {
            audioContentCuration.description = request.description
        }

        if (request.isAdult != null) {
            audioContentCuration.isAdult = request.isAdult
        }

        if (request.isActive != null) {
            audioContentCuration.isActive = request.isActive
        }

        if (request.isSeries != null) {
            audioContentCuration.isSeries = request.isSeries
        }

        if (request.tabId != null) {
            val tab = contentMainTabRepository.findByIdOrNull(request.tabId)

            if (tab != null) {
                audioContentCuration.tab = tab
            }
        }
    }

    @Transactional
    fun updateContentCurationOrders(ids: List<Long>) {
        for (index in ids.indices) {
            val audioContentCuration = repository.findByIdOrNull(ids[index])

            if (audioContentCuration != null) {
                audioContentCuration.orders = index + 1
            }
        }
    }

    fun getContentCurationList(tabId: Long): List<GetAdminContentCurationResponse> {
        return repository.getAudioContentCurationList(tabId = tabId)
    }

    fun getCurationItem(curationId: Long): List<GetCurationItemResponse> {
        val curation = repository.findByIdOrNull(curationId)
            ?: throw SodaException("잘못된 요청입니다.")

        return if (curation.isSeries) {
            contentCurationItemRepository.getAudioContentCurationSeriesItemList(curationId)
        } else {
            contentCurationItemRepository.getAudioContentCurationItemList(curationId)
        }
    }

    fun searchCurationContentItem(curationId: Long, searchWord: String): List<SearchCurationItemResponse> {
        return repository.searchCurationContentItem(curationId, searchWord)
    }

    fun searchCurationSeriesItem(curationId: Long, searchWord: String): List<SearchCurationItemResponse> {
        return repository.searchCurationSeriesItem(curationId, searchWord)
    }

    @Transactional
    fun addItemToCuration(request: AddItemToCurationRequest) {
        // 큐레이션 조회
        val audioContentCuration = repository.findByIdOrNull(id = request.curationId)
            ?: throw SodaException("잘못된 요청입니다.")

        if (audioContentCuration.isSeries) {
            request.itemIdList.forEach { seriesId ->
                val series = seriesRepository.findByIdAndActiveTrue(seriesId)

                if (series != null) {
                    val item = contentCurationItemRepository.findByCurationIdAndSeriesId(
                        curationId = request.curationId,
                        seriesId = series.id
                    ) ?: AudioContentCurationItem()
                    item.curation = audioContentCuration
                    item.series = series
                    item.isActive = true
                    contentCurationItemRepository.save(item)
                }
            }
        } else {
            request.itemIdList.forEach { contentId ->
                val audioContent = contentRepository.findByIdAndActiveTrue(contentId)

                if (audioContent != null) {
                    val item = contentCurationItemRepository.findByCurationIdAndContentId(
                        curationId = request.curationId,
                        contentId = audioContent.id
                    ) ?: AudioContentCurationItem()
                    item.curation = audioContentCuration
                    item.content = audioContent
                    item.isActive = true
                    contentCurationItemRepository.save(item)
                }
            }
        }
    }

    @Transactional
    fun removeItemInCuration(request: RemoveItemInCurationRequest) {
        val audioContentCurationItem = contentCurationItemRepository.findByCurationIdAndItemId(
            curationId = request.curationId,
            itemId = request.itemId
        )

        audioContentCurationItem?.isActive = false
    }

    @Transactional
    fun updateItemInCurationOrders(request: UpdateCurationItemOrdersRequest) {
        val ids = request.itemIds
        for (index in ids.indices) {
            val item = contentCurationItemRepository.findByCurationIdAndItemId(
                curationId = request.curationId,
                itemId = ids[index]
            )

            if (item != null) {
                item.orders = index + 1
            }
        }
    }
}
