package kr.co.vividnext.sodalive.admin.statistics.member

import com.querydsl.core.types.dsl.DateTimePath
import com.querydsl.core.types.dsl.Expressions
import com.querydsl.core.types.dsl.StringTemplate
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.can.charge.ChargeStatus
import kr.co.vividnext.sodalive.can.charge.QCharge.charge
import kr.co.vividnext.sodalive.can.payment.PaymentStatus
import kr.co.vividnext.sodalive.can.payment.QPayment.payment
import kr.co.vividnext.sodalive.member.QMember.member
import kr.co.vividnext.sodalive.member.QSignOut.signOut
import org.springframework.stereotype.Repository
import java.time.LocalDateTime

@Repository
class AdminMemberStatisticsRepository(private val queryFactory: JPAQueryFactory) {
    fun getTotalSignUpCount(startDate: LocalDateTime, endDate: LocalDateTime): Int {
        return queryFactory
            .select(member.id)
            .from(member)
            .where(
                member.createdAt.goe(startDate),
                member.createdAt.loe(endDate)
            )
            .fetch()
            .size
    }

    fun getTotalSignOutCount(startDate: LocalDateTime, endDate: LocalDateTime): Int {
        return queryFactory
            .select(signOut.id)
            .from(signOut)
            .where(
                signOut.createdAt.goe(startDate),
                signOut.createdAt.loe(endDate)
            )
            .fetch()
            .size
    }

    fun getPaymentMemberCount(startDate: LocalDateTime, endDate: LocalDateTime): Int {
        return queryFactory
            .select(
                QDateAndMemberCount(
                    getFormattedDate(charge.createdAt),
                    member.id.countDistinct().castToNum(Int::class.java)
                )
            )
            .from(charge)
            .innerJoin(charge.member, member)
            .innerJoin(charge.payment, payment)
            .where(
                charge.status.eq(ChargeStatus.CHARGE),
                payment.status.eq(PaymentStatus.COMPLETE),
                charge.createdAt.goe(startDate),
                charge.createdAt.loe(endDate)
            )
            .groupBy(getFormattedDate(charge.createdAt))
            .fetch()
            .sumOf { it.memberCount }
    }

    fun getSignUpCountInRange(startDate: LocalDateTime, endDate: LocalDateTime): List<DateAndMemberCount> {
        return queryFactory
            .select(
                QDateAndMemberCount(
                    getFormattedDate(member.createdAt),
                    member.id.countDistinct().castToNum(Int::class.java)
                )
            )
            .from(member)
            .where(
                member.createdAt.goe(startDate),
                member.createdAt.loe(endDate)
            )
            .groupBy(getFormattedDate(member.createdAt))
            .orderBy(getFormattedDate(member.createdAt).desc())
            .fetch()
    }

    fun getSignOutCountInRange(startDate: LocalDateTime, endDate: LocalDateTime): List<DateAndMemberCount> {
        return queryFactory
            .select(
                QDateAndMemberCount(
                    getFormattedDate(signOut.createdAt),
                    signOut.id.countDistinct().castToNum(Int::class.java)
                )
            )
            .from(signOut)
            .where(
                signOut.createdAt.goe(startDate),
                signOut.createdAt.loe(endDate)
            )
            .groupBy(getFormattedDate(signOut.createdAt))
            .orderBy(getFormattedDate(signOut.createdAt).desc())
            .fetch()
    }

    fun getPaymentMemberCountInRange(startDate: LocalDateTime, endDate: LocalDateTime): List<DateAndMemberCount> {
        return queryFactory
            .select(
                QDateAndMemberCount(
                    getFormattedDate(charge.createdAt),
                    member.id.countDistinct().castToNum(Int::class.java)
                )
            )
            .from(charge)
            .innerJoin(charge.member, member)
            .innerJoin(charge.payment, payment)
            .where(
                charge.status.eq(ChargeStatus.CHARGE),
                payment.status.eq(PaymentStatus.COMPLETE),
                charge.createdAt.goe(startDate),
                charge.createdAt.loe(endDate)
            )
            .groupBy(getFormattedDate(charge.createdAt))
            .orderBy(getFormattedDate(charge.createdAt).desc())
            .fetch()
    }

    private fun getFormattedDate(dateTimePath: DateTimePath<LocalDateTime>): StringTemplate {
        return Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                dateTimePath,
                "UTC",
                "Asia/Seoul"
            ),
            "%Y-%m-%d"
        )
    }
}
