package kr.co.vividnext.sodalive.can.coupon

import com.fasterxml.jackson.databind.ObjectMapper
import kr.co.vividnext.sodalive.aws.sqs.SqsEvent
import kr.co.vividnext.sodalive.aws.sqs.SqsEventType
import kr.co.vividnext.sodalive.can.charge.ChargeService
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.MemberRepository
import org.apache.poi.xssf.usermodel.XSSFWorkbook
import org.springframework.context.ApplicationEventPublisher
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import java.io.ByteArrayInputStream
import java.io.ByteArrayOutputStream
import java.io.IOException
import java.time.LocalDateTime
import java.time.ZoneId
import java.time.format.DateTimeFormatter

@Service
class CanCouponService(
    private val issueService: CanCouponIssueService,
    private val chargeService: ChargeService,

    private val repository: CanCouponRepository,
    private val couponNumberRepository: CanCouponNumberRepository,

    private val memberRepository: MemberRepository,

    private val objectMapper: ObjectMapper,
    private val applicationEventPublisher: ApplicationEventPublisher
) {
    fun generateCoupon(request: GenerateCanCouponRequest) {
        val message = objectMapper.writeValueAsString(request)
        applicationEventPublisher.publishEvent(SqsEvent(type = SqsEventType.GENERATE_COUPON, message = message))
    }

    @Transactional
    fun modifyCoupon(request: ModifyCanCouponRequest) {
        issueService.checkAnyChanges(request)

        val canCoupon = repository.findByIdOrNull(id = request.couponId)
            ?: throw SodaException("잘못된 쿠폰번호입니다.\n고객센터로 문의해 주시기 바랍니다.")

        if (request.validity != null) {
            val dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")
            val validity = LocalDateTime.parse(request.validity, dateTimeFormatter)
                .atZone(ZoneId.of("Asia/Seoul"))
                .withZoneSameInstant(ZoneId.of("UTC"))
                .toLocalDateTime()

            if (validity <= canCoupon.validity) {
                throw SodaException("유효기간은 기존 유효기간 이후 날짜로 설정하실 수 있습니다.")
            }

            canCoupon.validity = validity
        }

        if (request.isActive != null) {
            canCoupon.isActive = request.isActive
        }

        if (request.isMultipleUse != null) {
            canCoupon.isMultipleUse = request.isMultipleUse
        }
    }

    fun getCouponList(offset: Long, limit: Long): GetCouponListResponse {
        val totalCount = repository.getCouponTotalCount()

        val items = repository.getCouponList(offset = offset, limit = limit)
            .asSequence()
            .map {
                val useCouponCount = couponNumberRepository.getUseCouponCount(id = it.id)
                it.useCouponCount = useCouponCount
                it
            }
            .toList()

        return GetCouponListResponse(totalCount, items)
    }

    fun getCouponNumberList(couponId: Long, offset: Long, limit: Long): GetCouponNumberListResponse {
        val totalCount = couponNumberRepository.getCouponNumberTotalCount(couponId = couponId)
        val items = couponNumberRepository.getCouponNumberList(couponId = couponId, offset = offset, limit = limit)
        return GetCouponNumberListResponse(totalCount, items)
    }

    fun downloadCouponNumberList(couponId: Long): ByteArrayInputStream {
        val header = listOf("순번", "쿠폰번호", "사용여부")
        val byteArrayOutputStream = ByteArrayOutputStream()
        val couponNumberList = couponNumberRepository.getAllCouponNumberList(couponId)

        val workbook = XSSFWorkbook()
        try {
            val sheet = workbook.createSheet()
            val row = sheet.createRow(0)
            header.forEachIndexed { index, string ->
                val cell = row.createCell(index)
                cell.setCellValue(string)
            }

            couponNumberList.forEachIndexed { index, item ->
                val couponNumberRow = sheet.createRow(index + 1)
                couponNumberRow.createCell(0).setCellValue(item.couponNumberId.toString())
                couponNumberRow.createCell(1).setCellValue(insertHyphens(item.couponNumber))
                couponNumberRow.createCell(2).setCellValue(
                    if (item.isUsed) {
                        "O"
                    } else {
                        "X"
                    }
                )
            }

            workbook.write(byteArrayOutputStream)
            return ByteArrayInputStream(byteArrayOutputStream.toByteArray())
        } catch (e: IOException) {
            throw SodaException("다운로드를 하지 못했습니다.\n다시 시도해 주세요.")
        } finally {
            workbook.close()
            byteArrayOutputStream.close()
        }
    }

    fun useCanCoupon(couponNumber: String, memberId: Long) {
        val member = memberRepository.findByIdOrNull(id = memberId)
            ?: throw SodaException("로그인 정보를 확인해주세요.")

        if (member.auth == null) throw SodaException("쿠폰은 본인인증을 하셔야 사용이 가능합니다.")

        issueService.validateAvailableUseCoupon(couponNumber, memberId)

        chargeService.chargeByCoupon(couponNumber, member)
    }

    private fun insertHyphens(input: String): String {
        return input.chunked(4).joinToString("-")
    }
}
