package kr.co.vividnext.sodalive.content.order

import com.querydsl.core.types.dsl.Expressions
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.main.GetAudioContentMainItem
import kr.co.vividnext.sodalive.content.main.QGetAudioContentMainItem
import kr.co.vividnext.sodalive.content.order.QOrder.order
import kr.co.vividnext.sodalive.content.playlist.ContentIdAndEndDateData
import kr.co.vividnext.sodalive.content.playlist.QContentIdAndEndDateData
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository
import java.time.Duration
import java.time.LocalDateTime

@Repository
interface OrderRepository : JpaRepository<Order, Long>, OrderQueryRepository

interface OrderQueryRepository {
    fun isExistOrdered(memberId: Long, contentId: Long): Boolean
    fun isExistOrderedAndOrderType(memberId: Long, contentId: Long): Pair<Boolean, OrderType?>
    fun getAudioContentRemainingTime(memberId: Long, contentId: Long, timezone: String): String
    fun getAudioContentOrderList(
        dateTime: LocalDateTime,
        coverImageHost: String,
        memberId: Long,
        offset: Long = 0,
        limit: Long = 10
    ): List<GetAudioContentOrderListItem>

    fun totalAudioContentOrderListCount(memberId: Long, dateTime: LocalDateTime): Int
    fun getAudioContentMainOrderList(
        dateTime: LocalDateTime,
        coverImageHost: String,
        memberId: Long,
        offset: Long = 0,
        limit: Long = 20
    ): List<GetAudioContentMainItem>

    fun findOrderedContent(contentIdList: List<Long>, memberId: Long): List<Long>
    fun findEndDateByContentId(contentIdList: List<Long>, memberId: Long): List<ContentIdAndEndDateData>
}

@Repository
class OrderQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : OrderQueryRepository {
    override fun isExistOrdered(memberId: Long, contentId: Long): Boolean {
        return queryFactory
            .select(order.id)
            .from(order)
            .where(
                order.member.id.eq(memberId)
                    .and(order.audioContent.id.eq(contentId))
                    .and(order.isActive.isTrue)
                    .and(
                        order.type.eq(OrderType.KEEP)
                            .or(
                                order.type.eq(OrderType.RENTAL)
                                    .and(order.endDate.after(LocalDateTime.now()))
                            )
                    )
            )
            .fetch()
            .isNotEmpty()
    }

    override fun isExistOrderedAndOrderType(memberId: Long, contentId: Long): Pair<Boolean, OrderType?> {
        val result = queryFactory
            .select(order.type)
            .from(order)
            .where(
                order.member.id.eq(memberId)
                    .and(order.audioContent.id.eq(contentId))
                    .and(order.isActive.isTrue)
                    .and(
                        order.type.eq(OrderType.KEEP)
                            .or(
                                order.type.eq(OrderType.RENTAL)
                                    .and(order.endDate.after(LocalDateTime.now()))
                            )
                    )
            )
            .fetch()

        val isExist = result.isNotEmpty()

        return if (!isExist) {
            Pair(false, null)
        } else {
            if (result.contains(OrderType.KEEP)) {
                Pair(true, OrderType.KEEP)
            } else {
                Pair(true, OrderType.RENTAL)
            }
        }
    }

    override fun getAudioContentRemainingTime(memberId: Long, contentId: Long, timezone: String): String {
        val result = queryFactory
            .select(order.endDate)
            .from(order)
            .where(
                order.member.id.eq(memberId)
                    .and(order.audioContent.id.eq(contentId))
                    .and(order.isActive.isTrue)
                    .and(
                        order.type.eq(OrderType.KEEP)
                            .or(
                                order.type.eq(OrderType.RENTAL)
                                    .and(order.endDate.after(LocalDateTime.now()))
                            )
                    )
            )
            .fetchFirst() ?: return ""

        val duration = Duration.between(LocalDateTime.now(), result)

        return duration.toHours().toString().padStart(2, '0') + "시간 " +
            duration.toMinutesPart().toString().padStart(2, '0') + "분"
    }

    override fun getAudioContentOrderList(
        dateTime: LocalDateTime,
        coverImageHost: String,
        memberId: Long,
        offset: Long,
        limit: Long
    ): List<GetAudioContentOrderListItem> {
        val where = order.member.id.eq(memberId)
            .and(order.isActive.isTrue)
            .and(
                order.type.eq(OrderType.KEEP)
                    .or(
                        order.type.eq(OrderType.RENTAL)
                            .and(order.startDate.before(dateTime))
                            .and(order.endDate.after(dateTime))
                    )
            )

        return queryFactory
            .select(
                QGetAudioContentOrderListItem(
                    audioContent.id,
                    audioContent.coverImage.prepend("/").prepend(coverImageHost),
                    audioContent.member.id,
                    audioContent.member.nickname,
                    audioContent.title,
                    audioContent.theme.theme,
                    audioContent.duration,
                    audioContent.isAdult,
                    order.type,
                    Expressions.ZERO,
                    Expressions.ZERO
                )
            )
            .from(order)
            .innerJoin(order.audioContent, audioContent)
            .where(where)
            .offset(offset)
            .limit(limit)
            .orderBy(order.createdAt.desc())
            .fetch()
    }

    override fun totalAudioContentOrderListCount(memberId: Long, dateTime: LocalDateTime): Int {
        val where = order.member.id.eq(memberId)
            .and(order.isActive.isTrue)
            .and(
                order.type.eq(OrderType.KEEP)
                    .or(
                        order.type.eq(OrderType.RENTAL)
                            .and(order.startDate.before(dateTime))
                            .and(order.endDate.after(dateTime))
                    )
            )

        return queryFactory.select(order.id)
            .from(order)
            .where(where)
            .fetch()
            .size
    }

    override fun getAudioContentMainOrderList(
        dateTime: LocalDateTime,
        coverImageHost: String,
        memberId: Long,
        offset: Long,
        limit: Long
    ): List<GetAudioContentMainItem> {
        return queryFactory
            .select(
                QGetAudioContentMainItem(
                    audioContent.id,
                    audioContent.coverImage.prepend("/").prepend(coverImageHost),
                    audioContent.title,
                    member.id,
                    member.profileImage
                        .prepend("/")
                        .prepend(coverImageHost),
                    member.nickname,
                    audioContent.price,
                    audioContent.duration
                )
            )
            .from(order)
            .innerJoin(order.audioContent, audioContent)
            .innerJoin(audioContent.member, member)
            .where(
                order.member.id.eq(memberId)
                    .and(order.isActive.isTrue)
                    .and(
                        order.type.eq(OrderType.KEEP)
                            .or(
                                order.type.eq(OrderType.RENTAL)
                                    .and(order.startDate.before(dateTime))
                                    .and(order.endDate.after(dateTime))
                            )
                    )
            )
            .offset(offset)
            .limit(limit)
            .orderBy(order.createdAt.desc())
            .fetch()
    }

    override fun findOrderedContent(contentIdList: List<Long>, memberId: Long): List<Long> {
        return queryFactory
            .select(audioContent.id)
            .from(order)
            .innerJoin(order.member, member)
            .innerJoin(order.audioContent, audioContent)
            .where(
                order.isActive.isTrue,
                member.id.eq(memberId),
                audioContent.id.`in`(contentIdList),
                order.endDate.isNull.or(order.endDate.after(LocalDateTime.now()))
            )
            .fetch()
    }

    override fun findEndDateByContentId(contentIdList: List<Long>, memberId: Long): List<ContentIdAndEndDateData> {
        return queryFactory
            .select(QContentIdAndEndDateData(audioContent.id, order.endDate))
            .from(order)
            .innerJoin(order.member, member)
            .innerJoin(order.audioContent, audioContent)
            .where(
                order.isActive.isTrue,
                member.id.eq(memberId),
                audioContent.id.`in`(contentIdList),
                order.endDate.isNull.or(order.endDate.after(LocalDateTime.now()))
            )
            .fetch()
    }
}
