package kr.co.vividnext.sodalive.creator.admin.calculate

import kr.co.vividnext.sodalive.admin.calculate.GetCalculateLiveResponse
import kr.co.vividnext.sodalive.admin.calculate.GetCumulativeSalesByContentResponse
import kr.co.vividnext.sodalive.extensions.convertLocalDateTime
import kr.co.vividnext.sodalive.member.Member
import org.springframework.cache.annotation.Cacheable
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class CreatorAdminCalculateService(private val repository: CreatorAdminCalculateQueryRepository) {
    @Transactional(readOnly = true)
    @Cacheable(
        cacheNames = ["default"],
        key = "'creatorCalculateLive:v20240403_01:' + " + "#member + ':' + #startDateStr + ':' + #endDateStr"
    )
    fun getCalculateLive(startDateStr: String, endDateStr: String, member: Member): List<GetCalculateLiveResponse> {
        val startDate = startDateStr.convertLocalDateTime()
        val endDate = endDateStr.convertLocalDateTime(hour = 23, minute = 59, second = 59)

        return repository
            .getCalculateLive(startDate, endDate, member.id!!)
            .map { it.toGetCalculateLiveResponse() }
    }

    @Transactional(readOnly = true)
    @Cacheable(
        cacheNames = ["cache_ttl_3_hours"],
        key = "'creatorCalculateContentList:v20240403_01:' + " +
            "#startDateStr + ':' + #endDateStr + ':' + #memberId + ':' + #offset + ':' + #limit"
    )
    fun getCalculateContentList(
        startDateStr: String,
        endDateStr: String,
        memberId: Long,
        offset: Long,
        limit: Long
    ): GetCalculateContentListResponse {
        val startDate = startDateStr.convertLocalDateTime()
        val endDate = endDateStr.convertLocalDateTime(hour = 23, minute = 59, second = 59)

        val totalCount = repository.getCalculateContentListTotalCount(startDate, endDate, memberId)
        val items = repository.getCalculateContentList(startDate, endDate, memberId, offset, limit)
            .map { it.toGetCalculateContentResponse() }

        return GetCalculateContentListResponse(totalCount, items)
    }

    @Transactional(readOnly = true)
    @Cacheable(
        cacheNames = ["cache_ttl_3_hours"],
        key = "'creatorCumulativeSalesByContent:' + " + "#memberId + ':' + #offset + ':' + #limit"
    )
    fun getCumulativeSalesByContent(memberId: Long, offset: Long, limit: Long): GetCumulativeSalesByContentResponse {
        val totalCount = repository.getCumulativeSalesByContentTotalCount(memberId)
        val items = repository
            .getCumulativeSalesByContent(memberId, offset, limit)
            .map { it.toCumulativeSalesByContentItem() }

        return GetCumulativeSalesByContentResponse(totalCount, items)
    }

    @Transactional(readOnly = true)
    @Cacheable(
        cacheNames = ["cache_ttl_3_hours"],
        key = "'creatorCalculateContentDonationList2:' + " +
            "#startDateStr + ':' + #endDateStr + ':' + #memberId + ':' + #offset + ':' + #limit"
    )
    fun getCalculateContentDonationList(
        startDateStr: String,
        endDateStr: String,
        memberId: Long,
        offset: Long,
        limit: Long
    ): GetCreatorCalculateContentDonationResponse {
        val startDate = startDateStr.convertLocalDateTime()
        val endDate = endDateStr.convertLocalDateTime(hour = 23, minute = 59, second = 59)

        val totalCount = repository.getCalculateContentDonationListTotalCount(startDate, endDate, memberId)
        val items = repository
            .getCalculateContentDonationList(startDate, endDate, memberId, offset, limit)
            .map { it.toGetCalculateContentDonationResponse() }

        return GetCreatorCalculateContentDonationResponse(totalCount, items)
    }

    @Transactional(readOnly = true)
    @Cacheable(
        cacheNames = ["cache_ttl_3_hours"],
        key = "'creatorCalculateCommunityPost:' + " +
            "#startDateStr + ':' + #endDateStr + ':' + #memberId + ':' + #offset + ':' + #limit"
    )
    fun getCalculateCommunityPost(
        startDateStr: String,
        endDateStr: String,
        memberId: Long,
        offset: Long,
        limit: Long
    ): GetCreatorCalculateCommunityPostResponse {
        val startDate = startDateStr.convertLocalDateTime()
        val endDate = endDateStr.convertLocalDateTime(hour = 23, minute = 59, second = 59)

        val totalCount = repository.getCalculateCommunityPostTotalCount(startDate, endDate, memberId)
        val items = repository
            .getCalculateCommunityPostList(startDate, endDate, memberId, offset, limit)
            .map { it.toGetCalculateCommunityPostResponse() }

        return GetCreatorCalculateCommunityPostResponse(totalCount, items)
    }
}
