package kr.co.vividnext.sodalive.explorer

import com.querydsl.core.types.Predicate
import com.querydsl.core.types.Projections
import com.querydsl.core.types.dsl.Expressions
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.can.use.CanUsage
import kr.co.vividnext.sodalive.can.use.QUseCan.useCan
import kr.co.vividnext.sodalive.can.use.QUseCanCalculate.useCanCalculate
import kr.co.vividnext.sodalive.can.use.UseCanCalculateStatus
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.explorer.QCreatorRanking.creatorRanking
import kr.co.vividnext.sodalive.explorer.follower.GetFollowerListDto
import kr.co.vividnext.sodalive.explorer.follower.QGetFollowerListDto
import kr.co.vividnext.sodalive.explorer.profile.ChannelNotice
import kr.co.vividnext.sodalive.explorer.profile.CreatorCheers
import kr.co.vividnext.sodalive.explorer.profile.QChannelNotice.channelNotice
import kr.co.vividnext.sodalive.explorer.profile.QCreatorCheers.creatorCheers
import kr.co.vividnext.sodalive.explorer.profile.TimeDifferenceResult
import kr.co.vividnext.sodalive.live.room.LiveRoom
import kr.co.vividnext.sodalive.live.room.LiveRoomType
import kr.co.vividnext.sodalive.live.room.QLiveRoom.liveRoom
import kr.co.vividnext.sodalive.live.room.cancel.QLiveRoomCancel.liveRoomCancel
import kr.co.vividnext.sodalive.live.room.visit.QLiveRoomVisit.liveRoomVisit
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.MemberRole
import kr.co.vividnext.sodalive.member.QMember
import kr.co.vividnext.sodalive.member.QMember.member
import kr.co.vividnext.sodalive.member.auth.QAuth.auth
import kr.co.vividnext.sodalive.member.following.QCreatorFollowing.creatorFollowing
import kr.co.vividnext.sodalive.member.tag.QCreatorTag.creatorTag
import kr.co.vividnext.sodalive.member.tag.QMemberCreatorTag.memberCreatorTag
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Repository
import java.time.Duration
import java.time.LocalDate
import java.time.LocalDateTime
import java.time.ZoneId
import java.time.format.DateTimeFormatter

@Repository
class ExplorerQueryRepository(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) {
    fun getCreatorFollowing(creatorId: Long, memberId: Long): GetCreatorFollowingResponse? {
        return queryFactory
            .select(QGetCreatorFollowingResponse(creatorFollowing.isActive, creatorFollowing.isNotify))
            .from(creatorFollowing)
            .where(
                creatorFollowing.isActive.isTrue
                    .and(creatorFollowing.creator.id.eq(creatorId))
                    .and(creatorFollowing.member.id.eq(memberId))
            )
            .fetchFirst()
    }

    fun getNotificationUserIds(creatorId: Long): List<Long> {
        return queryFactory
            .select(creatorFollowing.member.id)
            .from(creatorFollowing)
            .where(
                creatorFollowing.isActive.isTrue
                    .and(creatorFollowing.creator.id.eq(creatorId))
                    .and(creatorFollowing.member.isActive.isTrue)
            )
            .fetch()
    }

    fun getDonationCoinsDateRange(creatorId: Long, startDate: LocalDateTime, endDate: LocalDateTime): Int? {
        val donationCoin = useCanCalculate.can.sum()

        return queryFactory
            .select(donationCoin)
            .from(useCanCalculate)
            .innerJoin(useCanCalculate.useCan, useCan)
            .where(
                useCanCalculate.status.eq(UseCanCalculateStatus.RECEIVED)
                    .and(
                        useCan.canUsage.eq(CanUsage.DONATION)
                            .or(useCan.canUsage.eq(CanUsage.SPIN_ROULETTE))
                            .or(useCan.canUsage.eq(CanUsage.LIVE))
                    )
                    .and(useCan.isRefund.isFalse)
                    .and(useCanCalculate.recipientCreatorId.eq(creatorId))
                    .and(
                        useCanCalculate.createdAt.goe(
                            startDate
                                .atZone(ZoneId.of("Asia/Seoul"))
                                .withZoneSameInstant(ZoneId.of("UTC"))
                                .toLocalDateTime()
                        )
                    )
                    .and(
                        useCanCalculate.createdAt.lt(
                            endDate
                                .atZone(ZoneId.of("Asia/Seoul"))
                                .withZoneSameInstant(ZoneId.of("UTC"))
                                .toLocalDateTime()
                        )
                    )
            )
            .fetchOne()
    }

    fun getMemberDonationRankingTotal(creatorId: Long): Int {
        val userMember = QMember("user")

        val donation = useCan.rewardCan.add(useCan.can).sum()
        return queryFactory
            .select(userMember.id)
            .from(useCanCalculate)
            .innerJoin(useCanCalculate.useCan, useCan)
            .innerJoin(useCan.member, userMember)
            .where(
                useCan.isRefund.isFalse
                    .and(useCanCalculate.recipientCreatorId.eq(creatorId))
                    .and(
                        useCan.canUsage.eq(CanUsage.DONATION)
                            .or(useCan.canUsage.eq(CanUsage.SPIN_ROULETTE))
                            .or(useCan.canUsage.eq(CanUsage.LIVE))
                    )
            )
            .groupBy(useCan.member.id)
            .orderBy(donation.desc())
            .fetch()
            .size
    }

    fun getMemberDonationRanking(
        creatorId: Long,
        limit: Long,
        offset: Long = 0,
        withDonationCan: Boolean
    ): List<MemberDonationRankingResponse> {
        val member = QMember("user")

        val donation = useCan.rewardCan.add(useCan.can).sum()
        return queryFactory
            .select(member, donation)
            .from(useCanCalculate)
            .innerJoin(useCanCalculate.useCan, useCan)
            .innerJoin(useCan.member, member)
            .offset(offset)
            .limit(limit)
            .where(
                useCan.member.isActive.isTrue
                    .and(useCan.isRefund.isFalse)
                    .and(useCanCalculate.recipientCreatorId.eq(creatorId))
                    .and(
                        useCan.canUsage.eq(CanUsage.DONATION)
                            .or(useCan.canUsage.eq(CanUsage.SPIN_ROULETTE))
                            .or(useCan.canUsage.eq(CanUsage.LIVE))
                    )
            )
            .groupBy(useCan.member.id)
            .orderBy(donation.desc(), member.id.desc())
            .fetch()
            .asSequence()
            .map {
                val account = it.get(member)!!
                val donationCan = it.get(donation)!!
                MemberDonationRankingResponse(
                    account.id!!,
                    account.nickname,
                    if (account.profileImage != null) {
                        "$cloudFrontHost/${account.profileImage}"
                    } else {
                        "$cloudFrontHost/profile/default-profile.png"
                    },
                    if (withDonationCan) donationCan else 0
                )
            }
            .toList()
    }

    fun getNewCreators(): List<Member> {
        return queryFactory
            .selectFrom(member)
            .where(
                member.role.eq(MemberRole.CREATOR)
                    .and(member.createdAt.goe(LocalDateTime.now().minusDays(30)))
                    .and(member.isActive.isTrue)
            )
            .orderBy(Expressions.numberTemplate(Double::class.java, "function('rand')").asc())
            .limit(20)
            .fetch()
    }

    fun findCreatorByGender(gender: Int): List<Member> {
        val today = LocalDate.now()
        val offset = (today.dayOfMonth - 1) % 26
        val characters = ('A'..'Z').map { it.toString() }.plus((0..9).map { it.toString() })
        val rotatedChars = characters.drop(offset) + characters.take(offset)

        val caseWhenClauses = rotatedChars.withIndex().joinToString(" ") {
            "WHEN ${member.email} LIKE '${it.value}%' THEN ${it.index + 1}"
        }
        val caseExpression = Expressions.stringTemplate("CASE $caseWhenClauses END")

        val randomExpression = Expressions.numberTemplate(Double::class.java, "function('rand')").asc()

        return queryFactory
            .selectFrom(member)
            .innerJoin(member.auth, auth)
            .where(
                member.role.eq(MemberRole.CREATOR)
                    .and(member.isActive.isTrue)
                    .and(auth.gender.eq(gender))
                    .and(member.id.notIn(3, 351, 19209, 8103, 19775))
            )
            .orderBy(caseExpression.asc(), randomExpression)
            .offset(0)
            .limit(10)
            .fetch()
    }

    fun getSearchChannel(channel: String, memberId: Long): List<Member> {
        return queryFactory.selectFrom(member)
            .where(
                member.nickname.containsIgnoreCase(channel)
                    .and(member.isActive.isTrue)
                    .and(member.id.ne(memberId))
                    .and(member.role.eq(MemberRole.CREATOR))
            )
            .fetch()
    }

    fun getMember(memberId: Long): Member? {
        return queryFactory
            .selectFrom(member)
            .where(member.id.eq(memberId))
            .fetchFirst()
    }

    fun getSimilarCreatorList(creatorId: Long): List<SimilarCreatorResponse> {
        val creator = queryFactory
            .selectFrom(member)
            .where(member.id.eq(creatorId))
            .fetchFirst() ?: throw SodaException("없는 사용자 입니다.")

        val creatorTagCount = creator.tags
            .asSequence()
            .filter { it.tag.isActive }
            .map { it.tag }
            .toList().size

        if (creatorTagCount <= 0) {
            val where = member.role.eq(MemberRole.CREATOR)
                .and(member.id.ne(creatorId))
                .and(member.tags.size().gt(0))
                .and(memberCreatorTag.tag.isActive.isTrue)

            return getRandomCreatorList(where = where, limit = 3)
        }

        val cnt = memberCreatorTag.member.count()
        val similarCreators = queryFactory
            .select(member, cnt)
            .from(memberCreatorTag)
            .innerJoin(memberCreatorTag.member, member)
            .innerJoin(memberCreatorTag.tag, creatorTag)
            .leftJoin(member.auth, auth)
            .where(
                member.role.eq(MemberRole.CREATOR)
                    .and(member.id.ne(creatorId))
                    .and(
                        memberCreatorTag.tag.`in`(
                            creator.tags
                                .asSequence()
                                .filter { it.tag.isActive }
                                .map { it.tag }
                                .toList()
                        )
                    )
            )
            .groupBy(memberCreatorTag.member.id)
            .orderBy(cnt.desc())
            .limit(3)
            .fetch()
            .map {
                val account = it.get(member)!!
                SimilarCreatorResponse(
                    account.id!!,
                    account.nickname,
                    if (account.profileImage != null) {
                        "$cloudFrontHost/${account.profileImage}"
                    } else {
                        "$cloudFrontHost/profile/default-profile.png"
                    },
                    account.tags
                        .asSequence()
                        .filter { tag -> tag.tag.isActive }
                        .map { tag -> tag.tag.tag }
                        .toList()
                )
            }

        if (similarCreators.size < 3) {
            val userIds = similarCreators.map { it.userId }
            var where = member.role.eq(MemberRole.CREATOR)
                .and(member.id.ne(creatorId))
                .and(member.tags.size().gt(0))
                .and(memberCreatorTag.tag.isActive.isTrue)

            for (userId in userIds) {
                where = where.and(member.id.ne(userId))
            }

            val additionalCreator = getRandomCreatorList(where = where, limit = (3 - similarCreators.size).toLong())
            return similarCreators + additionalCreator
        } else {
            return similarCreators
        }
    }

    private fun getRandomCreatorList(where: Predicate, limit: Long): List<SimilarCreatorResponse> {
        val result = queryFactory
            .select(member)
            .from(memberCreatorTag)
            .join(memberCreatorTag.member, member)
            .innerJoin(memberCreatorTag.tag, creatorTag)
            .leftJoin(member.auth, auth)
            .groupBy(memberCreatorTag.member.id)
            .where(where)
            .limit(limit)
            .orderBy(Expressions.numberTemplate(Double::class.java, "function('rand')").asc())
            .fetch()
            .map {
                SimilarCreatorResponse(
                    it.id!!,
                    it.nickname,
                    if (it.profileImage != null) {
                        "$cloudFrontHost/${it.profileImage}"
                    } else {
                        "$cloudFrontHost/profile/default-profile.png"
                    },
                    it.tags
                        .asSequence()
                        .filter { tag -> tag.tag.isActive }
                        .map { tag -> tag.tag.tag }
                        .toList()
                )
            }
        return result
    }

    fun getLiveRoomList(
        creatorId: Long,
        userMember: Member,
        timezone: String,
        limit: Int,
        offset: Long = 0
    ): List<LiveRoomResponse> {
        var where = liveRoom.member.id.eq(creatorId)
            .and(liveRoom.cancel.id.isNull)
            .and(liveRoom.isActive.isTrue)

        if (userMember.auth == null) {
            where = where.and(liveRoom.isAdult.isFalse)
        }

        if (userMember.role == MemberRole.CREATOR) {
            where = where.and(liveRoom.isAvailableJoinCreator.isTrue)
        }

        val result = mutableListOf<LiveRoom>()

        if (offset == 0L) {
            result.addAll(
                queryFactory
                    .selectFrom(liveRoom)
                    .innerJoin(liveRoom.member, member)
                    .leftJoin(liveRoom.cancel, liveRoomCancel)
                    .where(where)
                    .orderBy(liveRoom.beginDateTime.asc())
                    .fetch()
            )
        }

        return result
            .map {
                val reservations = it.reservations
                    .filter { reservation ->
                        reservation.member!!.id!! == userMember.id!! && reservation.isActive
                    }

                val beginDateTime = it.beginDateTime
                    .atZone(ZoneId.of("UTC"))
                    .withZoneSameInstant(ZoneId.of(timezone))

                val isPaid = if (it.channelName != null) {
                    val useCan = queryFactory
                        .selectFrom(useCan)
                        .innerJoin(useCan.member, member)
                        .where(
                            useCan.member.id.eq(member.id)
                                .and(useCan.room.id.eq(it.id!!))
                                .and(useCan.canUsage.eq(CanUsage.LIVE))
                        )
                        .orderBy(useCan.id.desc())
                        .fetchFirst()

                    useCan != null
                } else {
                    false
                }

                LiveRoomResponse(
                    roomId = it.id!!,
                    title = it.title,
                    content = it.notice,
                    isPaid = isPaid,
                    beginDateTime = beginDateTime.format(
                        DateTimeFormatter.ofPattern("yyyy.MM.dd E hh:mm a")
                    ),
                    isAdult = it.isAdult,
                    price = it.price,
                    channelName = it.channelName,
                    managerNickname = it.member!!.nickname,
                    coverImageUrl = if (it.coverImage!!.startsWith("https://")) {
                        it.coverImage!!
                    } else {
                        "$cloudFrontHost/${it.coverImage!!}"
                    },
                    isReservation = reservations.isNotEmpty(),
                    isActive = it.isActive,
                    isPrivateRoom = it.type == LiveRoomType.PRIVATE
                )
            }
    }

    fun getNoticeString(creatorId: Long): String {
        return queryFactory
            .select(channelNotice.notice)
            .from(channelNotice)
            .where(channelNotice.member.id.eq(creatorId))
            .fetchFirst() ?: ""
    }

    fun getCheersList(creatorId: Long, timezone: String, offset: Long, limit: Long): GetCheersResponse {
        val totalCount = queryFactory
            .selectFrom(creatorCheers)
            .where(
                creatorCheers.creator.id.eq(creatorId)
                    .and(creatorCheers.isActive.isTrue)
                    .and(creatorCheers.parent.isNull)
            )
            .fetch()
            .count()

        val cheers = queryFactory
            .selectFrom(creatorCheers)
            .where(
                creatorCheers.creator.id.eq(creatorId)
                    .and(creatorCheers.isActive.isTrue)
                    .and(creatorCheers.parent.isNull)
            )
            .offset(offset)
            .limit(limit)
            .orderBy(creatorCheers.id.desc())
            .fetch()
            .asSequence()
            .map {
                val date = it.createdAt!!
                    .atZone(ZoneId.of("UTC"))
                    .withZoneSameInstant(ZoneId.of(timezone))

                GetCheersResponseItem(
                    cheersId = it.id!!,
                    memberId = it.member!!.id!!,
                    nickname = it.member!!.nickname,
                    profileUrl = if (it.member!!.profileImage != null) {
                        "$cloudFrontHost/${it.member!!.profileImage}"
                    } else {
                        "$cloudFrontHost/profile/default-profile.png"
                    },
                    content = it.cheers,
                    date = date.format(DateTimeFormatter.ofPattern("yyyy.MM.dd E hh:mm a")),
                    replyList = it.children.asSequence()
                        .map { cheers ->
                            val replyDate = cheers.createdAt!!
                                .atZone(ZoneId.of("UTC"))
                                .withZoneSameInstant(ZoneId.of(timezone))

                            GetCheersResponseItem(
                                cheersId = cheers.id!!,
                                memberId = cheers.member!!.id!!,
                                nickname = cheers.member!!.nickname,
                                profileUrl = if (cheers.member!!.profileImage != null) {
                                    "$cloudFrontHost/${cheers.member!!.profileImage}"
                                } else {
                                    "$cloudFrontHost/profile/default-profile.png"
                                },
                                content = cheers.cheers,
                                date = replyDate.format(DateTimeFormatter.ofPattern("yyyy.MM.dd E hh:mm a")),
                                replyList = listOf()
                            )
                        }
                        .toList()
                )
            }
            .toList()

        return GetCheersResponse(
            totalCount = totalCount,
            cheers = cheers
        )
    }

    fun getLiveCount(creatorId: Long): Long? {
        return queryFactory
            .select(liveRoom.id.count())
            .from(liveRoom)
            .where(
                liveRoom.member.id.eq(creatorId)
                    .and(liveRoom.channelName.isNotNull)
            )
            .fetchFirst()
    }

    fun getLiveTime(creatorId: Long): Long {
        val diffs = queryFactory
            .select(
                Projections.constructor(
                    TimeDifferenceResult::class.java,
                    liveRoom.beginDateTime,
                    liveRoom.updatedAt
                )
            )
            .from(liveRoom)
            .where(
                liveRoom.member.id.eq(creatorId)
                    .and(liveRoom.channelName.isNotNull)
            )
            .fetch()

        return diffs
            .asSequence()
            .map { Duration.between(it.beginDateTime, it.updatedAt).toSeconds() }
            .sum() / 3600
    }

    fun getLiveContributorCount(creatorId: Long): Long? {
        return queryFactory
            .select(liveRoomVisit.member.count())
            .from(liveRoomVisit)
            .innerJoin(liveRoomVisit.room, liveRoom)
            .where(
                liveRoom.member.id.eq(creatorId)
                    .and(liveRoom.channelName.isNotNull)
            )
            .fetchFirst()
    }

    fun getFollowerListTotalCount(creatorId: Long): Int {
        return queryFactory.select(creatorFollowing.id)
            .from(creatorFollowing)
            .innerJoin(creatorFollowing.member, member)
            .where(
                member.isActive.isTrue
                    .and(creatorFollowing.isActive.isTrue)
                    .and(creatorFollowing.creator.id.eq(creatorId))
                    .and(creatorFollowing.member.id.ne(creatorId))
            )
            .fetch()
            .size
    }

    fun getFollowerList(
        creatorId: Long,
        offset: Long,
        limit: Long
    ): List<GetFollowerListDto> {
        return queryFactory
            .select(
                QGetFollowerListDto(
                    member.id,
                    member.profileImage.prepend("/").prepend(cloudFrontHost),
                    member.nickname,
                    member.role
                )
            )
            .from(creatorFollowing)
            .innerJoin(creatorFollowing.member, member)
            .where(
                member.isActive.isTrue
                    .and(creatorFollowing.isActive.isTrue)
                    .and(creatorFollowing.creator.id.eq(creatorId))
                    .and(creatorFollowing.member.id.ne(creatorId))
            )
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    fun isFollow(creatorId: Long, memberId: Long): Boolean {
        return queryFactory
            .select(creatorFollowing.isActive)
            .from(creatorFollowing)
            .where(
                creatorFollowing.creator.id.eq(creatorId)
                    .and(creatorFollowing.member.id.eq(memberId))
            )
            .fetchOne() ?: false
    }

    fun getCreatorCheers(cheersId: Long): CreatorCheers? {
        return queryFactory
            .selectFrom(creatorCheers)
            .where(creatorCheers.id.eq(cheersId))
            .fetchFirst()
    }

    fun getCheers(cheersId: Long): CreatorCheers? {
        return queryFactory
            .selectFrom(creatorCheers)
            .where(creatorCheers.id.eq(cheersId))
            .fetchFirst()
    }

    fun getNotice(creatorId: Long): ChannelNotice? {
        return queryFactory
            .selectFrom(channelNotice)
            .where(channelNotice.member.id.eq(creatorId))
            .fetchFirst()
    }

    fun getContentCount(creatorId: Long): Long? {
        return queryFactory
            .select(audioContent.id.count())
            .from(audioContent)
            .where(
                audioContent.isActive.isTrue
                    .and(audioContent.member.id.eq(creatorId))
            )
            .fetchFirst()
    }

    fun getVisibleDonationRank(creatorId: Long): Boolean {
        return queryFactory
            .select(member.isVisibleDonationRank)
            .from(member)
            .where(member.id.eq(creatorId))
            .fetchFirst()
    }

    fun getCreatorRankings(): List<Member> {
        return queryFactory
            .select(member)
            .from(creatorRanking)
            .innerJoin(creatorRanking.member, member)
            .orderBy(creatorRanking.ranking.asc())
            .fetch()
    }
}
